<?php
/*
Plugin Name: RPS Include Content
Plugin URI: http://redpixel.com/
Description: Adds the ability to include content on the current post or page from another. 
Version: 1.0.15
Author: Red Pixel Studios
Author URI: http://redpixel.com/
License: GPL3
*/

/* 	Copyright (C) 2012 - 2013  Red Pixel Studios  (email : support@redpixel.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Adds the ability to include content on the current post or page from another.
 *
 * @package rps-include-content
 * @author Red Pixel Studios
 * @version 1.0.15
 * @todo Provide option for removing the hover effect by role.
 * @todo Allow rps-include shortcodes to remain even if other shortcodes are stripped.
 */
 
if ( ! class_exists( 'RPS_Include_Content', false ) ) :

class RPS_Include_Content {

	public function __construct() {
		add_action( 'init', array( &$this, 'cb_init' ) );
		add_action( 'wp_enqueue_scripts', array( &$this, 'cb_enqueue_styles_scripts' ) );
	}

	public function cb_init() {
		add_shortcode( 'rps-include', array( &$this, 'cb_include_shortcode' ) );
		add_shortcode( 'rps-include-content', array( &$this, 'cb_include_shortcode' ) );
		
		wp_register_style( 'rps-include-content', plugins_url( 'rps-include-content.css', __FILE__ ), false, '1.0.0' );
	}

	public function cb_enqueue_styles_scripts() {
		if ( current_user_can( 'edit_pages' ) || current_user_can( 'edit_posts' ) )
			wp_enqueue_style( 'rps-include-content' );
	}
		
	public function cb_include_shortcode( $atts, $content = null ) {
		// specify allowed values for shortcode attributes
		$allowed_titletag = array(
			'h1',
			'h2',
			'h3',
			'h4',
			'h5',
			'h6'
		);

		$allowed_content = array(
			'content',
			'excerpt',
			'none'
		);
		
		// specify defaults for shortcode attributes
		$defaults = array(
			'blog' => 1,
			'post' => 0,
			'page' => 0,
			'title' => false,
			'titletag' => 'h2',
			'titlelink' => false,
			'content' => 'content',
			'filter' => false,
			'shortcodes' => true
		);

		extract( shortcode_atts( $defaults, $atts ) );
		
		// convert string values to lowercase and trim
		$title = trim( strtolower( $title ) );
		$titletag = trim( strtolower( $titletag ) );
		$content = trim( strtolower( $content ) );

		// type cast strings as necessary
		$blog = absint( $blog );
		$post = absint( $post );
		$page = absint( $page );
		$title = ( $title == 'true' ) ? true : false;
		$titlelink = ( $titlelink == 'true' ) ? true : false;
		$filter = ( $filter == 'true' ) ? true : false;
		$shortcodes = ( $shortcodes == 'true' ) ? true : false;
		
		// handle if page attribute used instead of post
		$post = ( $post === 0 && $page !== 0 ) ? $page : $post;

		// test for allowed values
		if ( !in_array( $titletag, $allowed_titletag ) ) $titletag = $defaults['titletag'];
		if ( !in_array( $content, $allowed_content ) ) $content = $defaults['content'];

		if ( $post === 0 )
			return $this->error_msg( 'Post must be a non-zero integer.' );

		// add the calling blog/post to the array
		if ( empty( self::$included ) ) :
			$current_blog_info = $this->get_current_blog_info();
			self::$included[] = $current_blog_info['blog'] . ',' . $current_blog_info['post'];
		endif;

		$the_post = ( is_multisite() ) ? get_blog_post( $blog, $post ) : get_post( $post );

		if ( in_array( $blog . ',' . $post, self::$included ) ) :
			$i = 0;
			$looped_list = '<ol>';
			foreach( self::$included as $looped ) {
				$looped_blog_post = explode( ',', $looped );
				$looped_blog_id = $looped_blog_post[0];
				$looped_post_id = $looped_blog_post[1];
				$looped_blog = ( is_multisite() ) ? get_blog_details( $looped_blog_id )->blogname : get_bloginfo( 'name' );
				$looped_post = ( is_multisite() ) ? get_blog_post( $looped_blog_id, $looped_post_id  ) : get_post( $looped_post_id );
				$looped_post_permalink = ( is_multisite() ) ? get_blog_permalink( $looped_blog_id, $looped_post_id ) : get_permalink( $looped_post_id );

				if ( $i == 0 ) :
					$looped_list .= '<li>' . $looped_blog . ' &raquo; ' . $looped_post->post_title . ' (this ' . get_post_type( $looped_post_id ) . ')</li>';
				else :
					$looped_list .= '<li><a href="' . $looped_post_permalink . '">'. $looped_blog . ' &raquo; ' . $looped_post->post_title . '</a></li>';
				endif;

				$i++;
			}
			$looped_list .= '</ol>';
			return $this->error_msg( '<strong>An infinite content loop detected involving the following:</strong>' . $looped_list );
		endif;

		if ( $the_post == null ) 
			return $this->error_msg( 'Blog <strong>' . $blog . '</strong>:Post <strong>' . $post . '</strong> could not be found.' );

		$the_blog = ( is_multisite() ) ? get_blog_details( $blog )->blogname : get_bloginfo( 'name' );
		
		$the_permalink = ( is_multisite() ) ? get_blog_permalink( $blog, $post ) : get_permalink( $post );
		$the_edit_link = ( is_multisite() ) ? $this->get_edit_post_link( $post, $blog ) : $this->get_edit_post_link( $post );
		
		$the_header = ( count( self::$included ) === 1 ) ? '<div class="rps-include-content-label" title="Included Content">i</div><div class="rps-include-content-links"><a href="' . $the_permalink . '" title="View ' . $the_blog . ' &raquo; ' . $the_post->post_title . '">View</a><a href="' . $the_edit_link . '" title="Edit ' . $the_blog . ' &raquo; ' . $the_post->post_title . '">Edit</a></div>' : '';

		self::$included[] = $blog . ',' . $post;
		
		/**
		 * If the title is set to show then wrap it in the selected tag.
		 * If the titlelink is set to show then add it.
		 */
		$the_title = '';
		
		/**
		 * If the content is set to none then set the title to true since we need to show something.
		 */
		if ( $content == 'none' )
			$title = true;
		
		if ( $title and ! $titlelink ) :
			$the_title = '<' . $titletag . '>' . $the_post->post_title . '</' . $titletag . '>';
		elseif ( $title and $titlelink ) :
			$the_title = '<' . $titletag . '><a href="' . $the_permalink . '">' . $the_post->post_title . '</a></' . $titletag . '>';
		endif;
		
		/**
		 * Check to see if the post content is password protected and if so display the password form.
		 * A logged in user will also see the password form since that is the default behavior. If the
		 * content argument is set to excerpt then display the post_excerpt, if none then nothing, otherwise the post_content.
		 */
		if ( post_password_required( $the_post ) and $content != 'none' ) :
			$the_article = get_the_password_form();
		elseif ( $content == 'excerpt' ) :
			$the_article = $the_post->post_excerpt;
		elseif ( $content == 'none' ) :
			$the_article = '';
		else :
			$the_article = $the_post->post_content;
		endif;
		
		// Strip shortcodes if set to false.
		if ( ! $shortcodes ) $the_article = strip_shortcodes( $the_article );
				
		// Assemble the content to be output by merging the title with the selected post parts.
		$the_content = $the_title;
		$the_content .= ( $filter ) ? apply_filters( 'the_content', $the_article ) : do_shortcode( $the_article );
		
		array_pop( self::$included );
		
		return ( current_user_can( 'edit_pages' ) || current_user_can( 'edit_posts') ) ? $this->output_msg( $the_content, 'rps-include-content-source', $the_header ) : $the_content;
	}
	
	private function get_current_blog_info() {
		global $post;
		$blog_id = ( function_exists( 'get_current_blog_id' ) ) ? get_current_blog_id() : 1;
		$post_id = $post->ID;
		return array( 'blog' => $blog_id, 'post' => $post_id );
	}
	
	private function get_edit_post_link( $post, $blog = '' ) {
		$the_edit_post_path = 'post.php?post=' . $post . '&amp;action=edit';
		$the_admin_url = get_admin_url( $blog );
		return $the_admin_url . $the_edit_post_path;
	}
	
	private function error_msg( $e ) {
		return ( current_user_can( 'edit_pages' ) || current_user_can( 'edit_posts') ) ? $this->output_msg( $e, 'rps-include-content-source rps-error' ) : '';
	}
	
	private function output_msg( $content, $classes = '', $header = '' ) {
		$the_output = '<div class="' . esc_attr( $classes ) . '">' . $header . '<div class="rps-include-content">';
		$the_output .= $content;
		$the_output .= '</div></div>';
		return $the_output;
	}
	
	static private $included = array();

}
if ( ! isset( $rps_include_content ) ) $rps_include_content = new RPS_Include_Content;

endif;

?>